/***********************************************************************************
* QCron
* Copyright (C) 2009 Marcin Baszczewski <marcin@baszczewski.pl>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QMessageBox>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindowClass)
{
    ui->setupUi(this);

    QIcon icon = QIcon(":/icons/qcron.png");
    setWindowIcon(icon);

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setToolTip("QCron");
    trayIcon->setIcon(icon);
    trayIcon->show();
    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));



    trayIconMenu = new QMenu(this);

    quitAction = new QAction(tr("&Quit"), this);
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
    trayIconMenu->addAction(quitAction);

    trayIcon->setContextMenu(trayIconMenu);

    connect(qApp,SIGNAL(aboutToQuit()),this,SLOT(saveSettings()));

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));
    timer->setInterval(1000);
    loadSettings();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::saveSettings()
{
    qDebug("save settings");
    QSettings settings;
    settings.beginWriteArray("elements");
    for (int i = 0; i < elements.size(); ++i)
    {
        settings.setArrayIndex(i);
        settings.setValue("host", elements.at(i)->host);
        settings.setValue("duration", elements.at(i)->duration);
    }
    settings.endArray();

    if (ui->option1->checkState()==Qt::Checked)
    settings.setValue("option1", true);
    else
    settings.setValue("option1", false);

    if (ui->option2->checkState()==Qt::Checked)
    settings.setValue("option2", true);
    else
    settings.setValue("option2", false);
}

void MainWindow::loadSettings()
{
    qDebug("load settings");
    QSettings settings;
    int size = settings.beginReadArray("elements");

    for (int i = 0; i < size; ++i)
    {
        settings.setArrayIndex(i);
        Element *element = new Element;
        element->host = settings.value("host").toString();
        element->duration = settings.value("duration").toInt();
        element->elapsed = 0;
        elements.append(element);
    }
    settings.endArray();

    foreach (Element *element, elements)
    {
        QListWidgetItem *item = new QListWidgetItem();
        item->setText(element->host);
        item->setToolTip(QString().setNum(element->duration));
        this->ui->listWidget->addItem(item);
    }
    bool option1 = settings.value("option1").value<bool>();
    if (!option1);
    else
    {
        ui->option1->setChecked(true);
        timer->start();
        this->ui->state->setText("Stop");
    }
    bool option2 = settings.value("option2").value<bool>();
    if (!option2)
    show();
    else
    ui->option2->setChecked(true);
}

void MainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason)
    {
        case QSystemTrayIcon::Trigger:
            if (!this->isVisible())
            this->showNormal();
            else
            this->hide();
            break;
        default:;
    }
}

void MainWindow::on_insert_clicked()
{
    if (this->ui->host->text().isEmpty())
    {
        QMessageBox msgBox;
        msgBox.setText("Pusta strona?");
        msgBox.exec();
        return;
    }
    QListWidgetItem *item = new QListWidgetItem();
    item->setText(this->ui->host->text());
    item->setToolTip(QString().setNum(this->ui->duration->value()));
    this->ui->listWidget->addItem(item);
    Element *element = new Element;
    element->host = this->ui->host->text();
    element->duration = this->ui->duration->value();
    element->elapsed = 0;
    this->elements.append(element);

    this->ui->host->setText("");
    this->ui->duration->setValue(1);
}

void MainWindow::on_remove_clicked()
{
    if (this->ui->listWidget->selectedItems().count()<1)
    return;
    int index = this->ui->listWidget->row(this->ui->listWidget->selectedItems()[0]);
    if (index!=-1)
    {
        elements.removeAt(index);
        this->ui->listWidget->takeItem(index);
    }
}

void MainWindow::response(QString url)
{
    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
    connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
    connect(manager, SIGNAL(finished(QNetworkReply*)), manager, SLOT(deleteLater()));
    manager->get(QNetworkRequest(QUrl(url)));
}

void MainWindow::replyFinished(QNetworkReply *replay)
{
    qDebug(replay->url().toString().toAscii());
}

void MainWindow::timeout()
{
    foreach (Element *element, elements)
    {
        element->elapsed++;
        if (element->elapsed >= element->duration)
        {
            element->elapsed = 0;
            response(element->host);
        }
    }
}

void MainWindow::on_state_clicked()
{
    if (timer->isActive())
    {
        this->ui->state->setText("Start");
        timer->stop();
    }
    else
    {
        this->ui->state->setText("Stop");
        timer->start();
    }
}
